//
// Tesina di Elaborazione Numerica dei Segnali 1997-98 (Mian)
//
// Cappelletti Lorenzo	lency@dei.unipd.it
// Spangaro Raffaele	keatch@dei.unipd.it
//
//
// Progettazione e realizzazione di un filtro ellittico del 4o ordine 
//

// Costanti

Fc = 8000;		// frequenza di campionamento
T  = 1/Fc;
n  = [1:1000];		// numero di campioni del segnale in ingresso
z  = poly(0, 'z');


// Definizione del segnale di ingresso
fp = 0500;		// frequenza in banda passante
ft = 2500;		// frequenza in banda transitoria
fa = 3000;		// frequenza in banda attenuata

x  = sin(2*%pi*fp*n*T) + sin(2*%pi*ft*n*T) + 10*sin(2*%pi*fa*n*T);

xbasc();
X  = abs(fft(x,-1));
plot2d(4*2*[1:500]*1E-3, X([1:500]), 1, "011", "X", [0, 0, 4, 6E3], [2, 8, 2, 6] );
xtitle(" ", "f (kHz)", "X(f)");
halt();


// Coefficenti della realizzazione parallelo forniti lanciando il programma
// "ellittic" con i seguenti dati:
// 	nessuna normalizzazione
// 	N   = 4
// 	DBR = .1
// 	Fs  = 8000
// 	F1  = 0
// 	F2  = 2000
// 	F3  = 3000


b  = [ -.14955E+00  -.22324E+00;
       -.76744E+00   .62386E+00 ];

a  = [  .22117E+00   .66501E+00;
       -.20190E+00   .12483E+00 ];

G  = .15284E+01;


H  = syslin('d', ...
							      G * ( 1 + ...
   ( b(1,1) + b(1,2)*z^(-1) ) / ( 1 + a(1,1)*z^(-1) + a(1,2)*z^(-2) ) + ...
   ( b(2,1) + b(2,2)*z^(-1) ) / ( 1 + a(2,1)*z^(-1) + a(2,2)*z^(-2) ) ) ...
)

xbasc();
plzr(H);
halt();

[frq, Hejl] = repfreq(H, .005, 4000/Fc, .005); // calcolo di H(e^(jl))
mH = 20*log10(abs( Hejl ));


// Definizione della funzione arrotondamento
deff( '[y] = Q(x)', [
   'y  = round(x*2^bit)/2^bit'
])


// Funzione di trasferimento quantizzata a 15 bit.
bit= 15;		// numero di bit per l'arrotondamento
b15= Q(b);
a15= Q(a);
G15= Q(G);

H15 = syslin('d', ...
							      G15 * ( 1 + ...
   ( b15(1,1)+b15(1,2)*z^(-1) ) / ( 1+a15(1,1)*z^(-1)+a15(1,2)*z^(-2) ) + ...
   ( b15(2,1)+b15(2,2)*z^(-1) ) / ( 1+a15(2,1)*z^(-1)+a15(2,2)*z^(-2) ) ) ...
)

[frq15, Hejl] = repfreq(H15, .005, 4000/Fc, .005);
mH15 = 20*log10(abs( Hejl ));


// Funzione di trasferimento quantizzata a 7 bit.
bit= 7;
b7 = Q(b);
a7 = Q(a);
G7 = Q(G);

H7 = syslin('d', ...
								 G7 * ( 1 + ...
   ( b7(1,1) + b7(1,2)*z^(-1) ) / ( 1 + a7(1,1)*z^(-1) + a7(1,2)*z^(-2) ) + ...
   ( b7(2,1) + b7(2,2)*z^(-1) ) / ( 1 + a7(2,1)*z^(-1) + a7(2,2)*z^(-2) ) ) ...
)

[frq7, Hejl] = repfreq(H7, .005, 4000/Fc, .005);
mH7 = 20*log10(abs( Hejl ));


// Plotting delle varie f.d.t.
xbasc();
plot2d1("onn", [frq*Fc]'/1000, [mH; mH15; mH7]', [1, -1, -4], "111", "precisione infinita@15+1 bit@7+1 bit", [0, -80, 4, 10], [2, 8, 2, 9]);
xtitle(" ", "f (kHz)", "|H| (dB)");
halt();

// Particolare in banda passante
xbasc();
r0 = zeros(frq);
r01= -.1*ones(frq);
plot2d1("onn", [frq*Fc]', [mH; mH15; mH7; r0; r01]', [1, -1, -4, 1, 1], "111", "precisione infinita@15+1 bit@7+1 bit", [0, -.125, 2000, .05], [2, 5, 2, 7]);
xtitle(" ", "f (Hz)", "|H| (dB)");


//  Caricamento dello stato del filtro.
W  = zeros(2,2);
Y  = zeros(1,2);	// uscita di ogni cella
y  = zeros(n);		// uscita del filtro


// Filtraggio a precisione infinita

for i = n

   for c = 1:2
      
      aux    = x(i) - a(c,1)*W(c,1) - a(c,2)*W(c,2);
      Y(c)   =        b(c,1)*aux    + b(c,2)*W(c,1);

      W(c,2) = W(c,1);	// aggiornamento dello stato
      W(c,1) = aux;

   end;

   y(i) = G*( x(i) + Y(1) + Y(2) ) ;

end;

xbasc();
Y  = abs(fft(y,-1));
plot2d(4*2*[1:500]*1E-3, Y([1:500]), 1, "011", "X", [0, 0, 4, 350], [2, 8, 2, 6] );
xtitle(" ", "f (kHz)", "Y(f)");
halt();


// Filtreggio a precisione finita

bit = 15;
y15 = zeros(n);
W   = zeros(2,2);
Y   = zeros(2,1);

for i = n

   for c = 1:2
      
      X      = Q(x(i));
      aux    = Q( X - a15(c,1)*W(c,1) - a15(c,2)*W(c,2) );
      Y(c)   = b15(c,1)*aux + b15(c,2)*W(c,1);

      W(c,2) = W(c,1);
      W(c,1) = aux;

   end;
 
   y15(i) = G15 * Q( X + Y(1) + Y(2) );

end;


bit= 7;
y7 = zeros(n);
W  = zeros(2,2);
Y  = zeros(2,1);

for i = n

   for c = 1:2
      
      X      = Q(x(i));
      aux    = Q( X - a7(c,1)*W(c,1) - a7(c,2)*W(c,2) );
      Y(c)   = b7(c,1)*aux + b7(c,2)*W(c,1);

      W(c,2) = W(c,1);
      W(c,1) = aux;

   end;
 
   y7(i) = G7 * Q( X + Y(1) + Y(2) ) ;

end;


xbasc();
npu= 49;
plot2d1("onn", n', [y; y15; y7; y]', [-9, -1, -4, 1], "111", "precisione infinita@15+1 bit@7+1 bit", [0, -2.5, npu+1, 2.5], [2, 5, 2, 5]);
xtitle(" ", "campioni", "ampiezza");
