*
*   File name:   lpefac3.asm
*
*   Description: filtro passa-basso ellittico realizzato in forma parallela.
*	Versione REAL-TIME con coefficienti SCALATI in norma l2.
*
*	Non e` facile riuscire a produrre un suono e sentirlo filtrato
*	all'uscita del convertitore. Pertanto, per ottenere una
*	convincente simulazione del corretto funzionamento del filtro,
*	procedere come segue:
*	- visualizzare un grafico a dominio frequenziale all'indirizzo
*	"inpp" con "Buffer size" e "Display size" pari a "length";
*	- visualizzare un grafico a dominio frequenziale all'indirizzo
*	"outpp" con "Buffer size" e "Display size" pari a "length";
*	- mandare in esecuzione il programma e fischiare nel microfono;
*	- fermare l'esecuzione durante il fischio.
*
*
*   Specifiche di progetto:
*
*		fp = 2 kHz	DBR < .5 dB
*		fs = 3 kHz	Att < 40 dB
*
*   Caratteristiche del filtro:
*
*		fp = 2 kHz	DBR < .1 dB
*		fs = 3 kHz	Att < 49 dB
*
*
*   Last mofified: 18/06/98
*

	.title   "Low pass elliptic filter: DBR = .1 dB, scaled, real-time, l2"
	.mmregs
	.width   80
	.length  60

	.setsect ".text",   1800h, 0
	.setsect ".data",   0200h, 1
	.setsect ".bss",    0210h, 1
	.setsect "vectors", 0180h, 0


	.sect "vectors"			; interrupt vector table stored at 
AC01_enable .set 1
	.copy "lpefvecs.asm"		; location 0x0180


	.sect ".data"
*
*	|------------------------------------->|
*	|                                      |
*	|                                      |
*   --->1--- 1/2 --->4----------->4--- b'10 -->2--- G --->3
*	|            |            |            |
*	|            |            |            |
*	|            |<-- -a11 ---5--- b'11 -->2
*	|            |            |            |
*	|            |            |            |
*	|            |<-- -a12 ---6            |
*	|                                      |
*	|                                      |
*	|----------->7----------->7--- b20 --->2
*	             |            |            |
*	             |            |            |
*	             |<-- -a21 ---8--- b21 --->|
*	             |            |
*	             |            |
*	             |<-- -a22 ---9
*

COEFF
	.word	-7316*2
	.word	21792
	.word	7248
	.word	-4900*2
	.word	20442
	.word	4090
	.word	-6616
	.word	-25148
	.word	25041
	; b11 a12 a11 b10
	; b21 a22 a21 b20
	; G/2

STATUS	.word	1	; W11
	.word	2	; W12
	.word	3	; W21
	.word	4	; W22

intmp	.word	0	; word temporaneo del dato in ingresso

ov4_counter		; contatore di overflow del nodo 4
	.word	0
ov7_counter		; contatore di overflow del nodo 7
	.word	0
ov2_counter		; contatore di overflow del nodo 2
	.word	0
end_data

inbuf	.set 2000h	; Per compatibilita` con la versione "not real-time"
length	.set 080h
outbuf	.set 2400h


	.sect ".text"
start:
	intm = 1		; disable all interrupts
	call AC01INIT		; initialize Analog interface.
	DP = #0
	pmst = #01a0h		; Interrupt pointer maps vectors to address 180h
	sp = #0ffah		; stack pointer located in Communications Kernal
	imr = #240h		; unmask TDM RINT and HPIINT(host port interface)
	FRCT = #1
	OVM = #0

	.asg intmp, in
	.asg AR1, inb
	.asg AR2, out
	.asg AR3, coeff
	.asg AR4, status
	.asg AR5, ovc
	inb = #inbuf
	out = #outbuf
	BK = #length
	status = #STATUS
	repeat(#3), A = 0	; resetta lo stato del filtro
	*status+ = A
	DP = #intmp
	intm = 0		; enable all interrupts

WAIT:
	goto WAIT

receive:
	coeff  = #COEFF
	status = #STATUS
	ovc = #ov4_counter
	A = trcv
	@in= A				; (estensione di segno)
	*inb+% = A
inpp
	B = @in<<(16-0)			; y(n)=x(n)

first_cell
        B += *coeff+ * *status+		; y(n)+=(2*b11)*W11
	A = @in<<(16-1)			; y'=x(n)/2
	A -= *coeff+ * *status-		; y'-=a22*W22
	A = rnd(A - *status * *coeff+)	; y'-=a21*W21 (T=W21)
	if (AOV) execute(1)
	   *ovc += 1
	*ovc+ += 0			; (serve solo per incrementare il puntatore)
	*status+ = hi(A)		; W21=y'
	*status+ = T			; W22=T
	B += *coeff+ * hi(A)		; y(n)+=(2*b20)*y'

second_cell
        B += *coeff+ * *status+		; y(n)+=b21*W21
	A = @in<<(16-0)			; y'=x(n)
	A -= *coeff+ * *status-		; y'-=a22*W22
	A = rnd(A - *status * *coeff+)	; y'-=a21*W21 (T=W21)
	if (AOV) execute(1)
	   *ovc += 1
	*ovc+ += 0			; (serve solo per incrementare il puntatore)
	*status+ = hi(A)		; W21=y'
	*status+ = T			; W22=T
	B = rnd(B + *coeff+ * hi(A))	; y(n)+=b20*y'

out_cells
	A = B				; y'=y(n)
	B = *coeff * hi(A)		; y(n)=(G/2)*y'
	B += *coeff * hi(A)		; y(n)+=(G/2)*y'
	if (BOV) execute(1)
	   *ovc += 1
	B = B<<(16-0)

	A = B & #~11b
	*out+% = A
outpp
	tdxr = A
	return_enable


transmit:
	return_enable


	.copy "acinit.asm"

	.end
